import React, {
  ReactNode,
  ComponentType,
  useState,
  useRef,
  useEffect,
} from "react";
import styles from "./OrganizedContent.module.css";

export interface LinkProps {
  className?: string;
  url: string;
  children: string | ReactNode | (string | ReactNode)[];
}

type Link = ComponentType<LinkProps>;

interface Heading {
  title: string;
  url: string;
  content: ReactNode;
}

interface Props {
  headings: Heading[];
  currentIndex: number;
  link: Link;
  children: ReactNode;
}

interface ChildProps {
  headings: Heading[];
  currentIndex: number;
  link: Link;
}

interface MobileProps {
  open: boolean;
  setOpen: React.Dispatch<React.SetStateAction<boolean>>;
  childProps: ChildProps;
}

export const OrganizedContent = ({
  headings,
  currentIndex,
  link: Link,
  children,
}: Props) => {
  const [open, setOpen] = useState(false);
  //const node = useRef();
  //useOnClickOutside(node, () => setOpen(false));

  const isReadAll = headings[currentIndex].title === "Read All";

  const readAllContent = headings
    .filter((heading: { title: string }) => heading.title !== "Read All")
    .map((heading) => (
      <div key={heading.url}>
        <h1>{heading.title}</h1>
        {heading.content}
      </div>
    ));

  const childProps: ChildProps = {
    headings: headings,
    currentIndex: currentIndex,
    link: Link,
  };

  const mobileProps: MobileProps = {
    open: open,
    setOpen: setOpen,
    childProps: childProps,
  };

  useEffect(() => {
    open
      ? (document.body.style.overflow = "hidden")
      : (document.body.style.overflow = "visible");
  }, [open]);

  return (
    <div className={styles.organizedContent}>
      <Nav {...childProps} />
      <div>
        {isReadAll ? (
          <>{readAllContent}</>
        ) : (
          <>
            <h1>{headings[currentIndex].title}</h1>
            {children}
            <Footer {...childProps} />
          </>
        )}
      </div>
      <MobileWrapper {...mobileProps} />
    </div>
  );
};

const Nav = ({ headings, currentIndex, link: Link }: ChildProps) => {
  return (
    <div className={styles.nav}>
      {headings.map((heading, index) => (
        <div
          className={index === currentIndex ? styles.selectedHeadingArea : ""}
          key={heading.url}
        >
          <div
            className={
              styles.navOption +
              " " +
              (heading.title === "Read All" ? styles.readAll : "")
            }
          >
            {index === currentIndex && (
              <span className={styles.selectedMarker} />
            )}
            <Link
              className={
                styles.navLink +
                " " +
                (index === currentIndex ? styles.navLinkSelected : "")
              }
              url={heading.url}
            >
              {heading.title}
            </Link>
          </div>
          <div className={styles.divider}></div>
        </div>
      ))}
    </div>
  );
};

const Footer = ({ headings, currentIndex, link: Link }: ChildProps) => {
  const prevHeading =
    currentIndex > 0 && headings[currentIndex - 1].title !== "Read All"
      ? headings[currentIndex - 1]
      : undefined;
  const nextHeading =
    currentIndex < headings.length - 1 &&
    headings[currentIndex + 1].title !== "Read All"
      ? headings[currentIndex + 1]
      : undefined;

  return (
    <div className={styles.footer}>
      {prevHeading && (
        <Link url={prevHeading.url}>
          <div className={styles.footerSection}>
            <svg
              xmlns="http://www.w3.org/2000/svg"
              width="14"
              height="9"
              viewBox="0 0 14 9"
              className={styles.arrow + " " + styles.prevArrow}
            >
              <path d="M6.24407 8.12713C6.64284 8.58759 7.35716 8.58759 7.75593 8.12713L13.3613 1.65465C13.9221 1.00701 13.4621 0 12.6053 0H1.39467C0.537918 0 0.0778675 1.00701 0.638743 1.65465L6.24407 8.12713Z" />
            </svg>
            <div>
              <div className={styles.prevNext}>Previous</div>
              <div className={styles.arrowHeading}>{prevHeading.title}</div>
            </div>
          </div>
        </Link>
      )}
      <div className={styles.footerDivider}></div>
      {nextHeading && (
        <Link url={nextHeading.url}>
          <div className={styles.footerSection}>
            <div>
              <div className={styles.prevNext + " " + styles.nextText}>
                Next
              </div>
              <div className={styles.arrowHeading}>{nextHeading.title}</div>
            </div>
            <svg
              xmlns="http://www.w3.org/2000/svg"
              width="14"
              height="9"
              viewBox="0 0 14 9"
              className={styles.arrow + " " + styles.nextArrow}
            >
              <path d="M6.24407 8.12713C6.64284 8.58759 7.35716 8.58759 7.75593 8.12713L13.3613 1.65465C13.9221 1.00701 13.4621 0 12.6053 0H1.39467C0.537918 0 0.0778675 1.00701 0.638743 1.65465L6.24407 8.12713Z" />
            </svg>
          </div>
        </Link>
      )}
    </div>
  );
};

const Burger = ({ open, setOpen }: MobileProps) => {
  const [prevScrollPos, setPrevScrollPos] = useState(0);
  const [visible, setVisible] = useState(true);
  const debouncedPrevScrollPos = useDebounce<number>(prevScrollPos, 100);

  useEffect(() => {
    const handleScroll = () => {
      // find current scroll position
      const currentScrollPos = window.pageYOffset;

      // set state based on location info (explained in more detail below)
      setVisible(
        (debouncedPrevScrollPos > currentScrollPos &&
          debouncedPrevScrollPos - currentScrollPos > 70) ||
          currentScrollPos < 10
      );

      // set state to new scroll position
      setPrevScrollPos(currentScrollPos);
    };
    window.addEventListener("scroll", handleScroll);

    return () => window.removeEventListener("scroll", handleScroll);
  }, [debouncedPrevScrollPos, visible]);

  const line1 = open ? styles.burgerLineOpen1 : styles.burgerLineClosed1;
  const line2 = open ? styles.burgerLineOpen2 : styles.burgerLineClosed2;
  const line3 = open ? styles.burgerLineOpen3 : styles.burgerLineClosed3; //BUG!!
  return (
    <div
      className={styles.burger + " " + (visible ? "" : styles.hiddenBurger)}
      onClick={() => setOpen(!open)}
    >
      <div className={line1} />
      <div className={line2} />
      <div className={line3} />
    </div>
  );
};

function MobileWrapper(mobileProps: MobileProps) {
  const wrapperRef = useRef<HTMLDivElement>(null);
  useOutsideAlerter(wrapperRef, mobileProps.setOpen);

  return (
    <div ref={wrapperRef}>
      <Burger {...mobileProps} />
      <Menu {...mobileProps} />
    </div>
  );
}

const Menu = ({ open, childProps }: MobileProps) => {
  const mobileNav = open
    ? styles.mobileNav
    : styles.mobileNav + " " + styles.mobileNavClosed;
  return (
    <div className={mobileNav}>
      <Nav {...childProps} />
    </div>
  );
};

function useOutsideAlerter(
  ref: React.RefObject<HTMLDivElement>,
  setOpen: React.Dispatch<React.SetStateAction<boolean>>
) {
  useEffect(() => {
    /**
     * Alert if clicked on outside of element
     */
    const handleClickOutside = (event: Event) => {
      if (ref.current && !ref.current.contains(event.target as Node)) {
        setOpen(false);
      }
    };

    // Bind the event listener
    document.addEventListener("mousedown", handleClickOutside);
    return () => {
      // Unbind the event listener on clean up
      document.removeEventListener("mousedown", handleClickOutside);
    };
  }, [ref, setOpen]);
}

function useDebounce<T>(value: T, delay?: number): T {
  const [debouncedValue, setDebouncedValue] = useState<T>(value);

  useEffect(() => {
    const timer = setTimeout(() => setDebouncedValue(value), delay || 500);

    return () => {
      clearTimeout(timer);
    };
  }, [value, delay]);

  return debouncedValue;
}
